@@ -57,70 +57,91 @@ static void PylibMC_ClientType_dealloc(PylibMC_Client *self) {
 
 static int PylibMC_Client_init(PylibMC_Client *self, PyObject *args,
         PyObject *kwds) {
-    PyObject *srv_list, *srv_list_it;
+    PyObject *srvs, *srvs_it, *c_srv;
+    unsigned char set_stype = 0, bin = 0;
 
-    if (!PyArg_ParseTuple(args, "O", &srv_list)) {
+    static char *kws[] = { "servers", "binary", NULL };
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|b", kws, &srvs, &bin)) {
+        return -1;
+    } else if ((srvs_it = PyObject_GetIter(srvs)) == NULL) {
         return -1;
-    } else {
-        kwds = NULL;
     }
 
-    if ((srv_list_it = PyObject_GetIter(srv_list)) != NULL) {
-        PyObject *c_srv;
+    memcached_behavior_set(self->mc, MEMCACHED_BEHAVIOR_BINARY_PROTOCOL, bin);
 
-        while ((c_srv = PyIter_Next(srv_list_it)) != NULL
-                && !PyErr_Occurred()) {
-            unsigned char stype;
-            char *hostname;
-            unsigned short int port;
+    while ((c_srv = PyIter_Next(srvs_it)) != NULL) {
+        unsigned char stype;
+        char *hostname;
+        unsigned short int port;
+        memcached_return rc;
 
-            port = 0;
-            if (PyString_Check(c_srv)) {
-                memcached_server_st *list;
+        port = 0;
+        if (PyString_Check(c_srv)) {
+            memcached_server_st *list;
 
-                list = memcached_servers_parse(PyString_AS_STRING(c_srv));
-                if (list != NULL) {
-                    memcached_return rc;
+            list = memcached_servers_parse(PyString_AS_STRING(c_srv));
+            if (list == NULL) {
+                PyErr_SetString(PylibMCExc_MemcachedError,
+                        "memcached_servers_parse returned NULL");
+                goto it_error;
+            }
 
-                    rc = memcached_server_push(self->mc, list);
-                    if (rc != MEMCACHED_SUCCESS) {
-                        PylibMC_ErrFromMemcached(self,
-                                "memcached_server_push", rc);
-                    }
-                    free(list);
-                } else {
-                    PyErr_SetString(PylibMCExc_MemcachedError,
-                            "memcached_servers_parse returned NULL");
-                }
-            } else if (PyArg_ParseTuple(c_srv, "Bs|H",
-                        &stype, &hostname, &port)) {
-                switch (stype) {
-                    case PYLIBMC_SERVER_TCP:
-                        memcached_server_add(self->mc, hostname, port);
-                        break;
-                    case PYLIBMC_SERVER_UDP:
-                        memcached_server_add_udp(self->mc, hostname, port);
-                        break;
-                    case PYLIBMC_SERVER_UNIX:
-                        if (port) {
-                            PyErr_SetString(PyExc_ValueError,
-                                    "can't set port on unix sockets");
-                        } else {
-                            memcached_server_add_unix_socket(
-                                    self->mc, hostname);
-                        }
-                        break;
-                    default:
-                        PyErr_Format(PyExc_ValueError,
-                                "bad server type: %u", stype);
+            rc = memcached_server_push(self->mc, list);
+            free(list);
+            if (rc != MEMCACHED_SUCCESS) {
+                PylibMC_ErrFromMemcached(self, "memcached_server_push", rc);
+                goto it_error;
+            }
+        } else if (PyArg_ParseTuple(c_srv, "Bs|H", &stype, &hostname, &port)) {
+            if (set_stype && set_stype != stype) {
+                PyErr_SetString(PyExc_ValueError, "can't mix transport types");
+                goto it_error;
+            } else {
+                set_stype = stype;
+                if (stype == PYLIBMC_SERVER_UDP) {
+                    memcached_behavior_set(self->mc,
+                        MEMCACHED_BEHAVIOR_USE_UDP, 1);
                 }
             }
-            Py_DECREF(c_srv);
+
+            switch (stype) {
+                case PYLIBMC_SERVER_TCP:
+                    rc = memcached_server_add(self->mc, hostname, port);
+                    break;
+                case PYLIBMC_SERVER_UDP:
+                    rc = memcached_server_add_udp(self->mc, hostname, port);
+                    break;
+                case PYLIBMC_SERVER_UNIX:
+                    if (port) {
+                        PyErr_SetString(PyExc_ValueError,
+                                "can't set port on unix sockets");
+                        goto it_error;
+                    }
+                    rc = memcached_server_add_unix_socket(self->mc, hostname);
+                    break;
+                default:
+                    PyErr_Format(PyExc_ValueError, "bad type: %u", stype);
+                    goto it_error;
+            }
+            if (rc != MEMCACHED_SUCCESS) {
+                PylibMC_ErrFromMemcached(self, "memcached_server_add_*", rc);
+                goto it_error;
+            }
         }
-        Py_DECREF(srv_list_it);
+        Py_DECREF(c_srv);
+        continue;
+
+it_error:
+        Py_DECREF(c_srv);
+        goto error;
     }
 
-    return PyErr_Occurred() ? -1 : 0;
+    Py_DECREF(srvs_it);
+    return 0;
+error:
+    Py_DECREF(srvs_it);
+    return -1;
 }
 
 static PyObject *_PylibMC_parse_memcached_value(char *value, size_t size,
@@ -331,17 +352,26 @@ static PyObject *_PylibMC_IncDec(PylibMC_Client *self, uint8_t dir,
     Py_ssize_t key_sz;
     unsigned int delta;
     uint64_t result;
+    memcached_return rc;
+    memcached_return (*incdec)(memcached_st *, const char *, size_t,
+                               unsigned int, uint64_t *);
 
     retval = NULL;
     delta = 1;
-    if (PyArg_ParseTuple(args, "s#|I", &key, &key_sz, &delta)
-            && _PylibMC_CheckKeyStringAndSize(key, key_sz)) {
-        memcached_return (*incdec)(memcached_st *, const char *, size_t,
-                unsigned int, uint64_t *);
-        incdec = (dir == PYLIBMC_INC) ? memcached_increment
-                                      : memcached_decrement;
-        incdec(self->mc, key, key_sz, delta, &result);
+    if (!PyArg_ParseTuple(args, "s#|I", &key, &key_sz, &delta)) {
+        return retval;
+    } else if (!_PylibMC_CheckKeyStringAndSize(key, key_sz)) {
+        return retval;
+    }
+
+    incdec = (dir == PYLIBMC_INC) ? memcached_increment : memcached_decrement;
+    rc = incdec(self->mc, key, key_sz, delta, &result);
+    if (rc == MEMCACHED_SUCCESS) {
         retval = PyLong_FromUnsignedLong((unsigned long)result);
+    } else {
+        char *fname = (dir == PYLIBMC_INC) ? "memcached_increment"
+                                           : "memcached_decrement";
+        PylibMC_ErrFromMemcached(self, fname, rc);
     }
 
     return retval;
@@ -440,7 +470,7 @@ static PyObject *PylibMC_Client_get_multi(PylibMC_Client *self, PyObject *args,
      */
     retval = PyDict_New();
 
-    if ((rc = memcached_mget(self->mc, keys, key_lens, nkeys))
+    if ((rc = memcached_mget(self->mc, (const char **)keys, key_lens, nkeys))
             != MEMCACHED_SUCCESS) {
         PylibMC_ErrFromMemcached(self, "memcached_mget", rc);
         goto cleanup;
@@ -691,15 +721,10 @@ static PyObject *PylibMC_Client_set_behaviors(PylibMC_Client *self,
 
         if (!PyMapping_HasKeyString(behaviors, b->name)) {
             continue;
-        }
-
-        v = PyMapping_GetItemString(behaviors, b->name);
-
-        if (v == NULL) {
+        } else if ((v = PyMapping_GetItemString(behaviors, b->name)) == NULL) {
             goto error;
         } else if (!PyInt_Check(v)) {
-            PyErr_Format(PyExc_TypeError, "behavior %s must be int",
-                         b->name);
+            PyErr_Format(PyExc_TypeError, "behavior %s must be int", b->name);
             goto error;
         }
 
@@ -745,6 +770,21 @@ static PyObject *PylibMC_Client_disconnect_all(PylibMC_Client *self) {
     memcached_quit(self->mc);
     Py_RETURN_NONE;
 }
+
+static PyObject *PylibMC_Client_clone(PylibMC_Client *self) {
+    PylibMC_Client *new_self;
+
+    /* XXX Is it really wise to short-circuit like this? I don't see a problem
+     * with it. */
+    new_self = (PylibMC_Client *)self->ob_type->tp_new(self->ob_type, NULL, NULL);
+    if (new_self == NULL) {
+        return NULL;
+    }
+    memset(new_self->mc, 0, sizeof(memcached_st));
+    new_self->mc = memcached_clone(new_self->mc, self->mc);
+
+    return (PyObject *)new_self;
+}
 /* }}} */
 
 static PyObject *PylibMC_ErrFromMemcached(PylibMC_Client *self, const char *what,
@@ -876,6 +916,8 @@ Oh, and: plankton.\n");
     PylibMCExc_MemcachedError = PyErr_NewException(
             "_pylibmc.MemcachedError", NULL, NULL);
 
+    PyModule_AddStringConstant(module, "__version__", PYLIBMC_VERSION);
+
     Py_INCREF(&PylibMC_ClientType);
     PyModule_AddObject(module, "client", (PyObject *)&PylibMC_ClientType);
 
