@@ -41,6 +41,8 @@
 #include <Python.h>
 #include <libmemcached/memcached.h>
 
+#include "pylibmc-version.h"
+
 /* Py_ssize_t appeared in Python 2.5. */
 #ifndef PY_SSIZE_T_MAX
 typedef ssize_t Py_ssize_t;
@@ -92,10 +94,16 @@ static PylibMC_Behavior PylibMC_behaviors[] = {
     { MEMCACHED_BEHAVIOR_RETRY_TIMEOUT, "retry_timeout" },
     { MEMCACHED_BEHAVIOR_KETAMA_WEIGHTED, "ketama_weighted" },
     { MEMCACHED_BEHAVIOR_KETAMA_HASH, "ketama_hash" },
-    { MEMCACHED_BEHAVIOR_BINARY_PROTOCOL, "binary_protocol" },
     { MEMCACHED_BEHAVIOR_SND_TIMEOUT, "snd_timeout" },
     { MEMCACHED_BEHAVIOR_RCV_TIMEOUT, "rcv_timeout" },
     { MEMCACHED_BEHAVIOR_SERVER_FAILURE_LIMIT, "server_failure_limit" },
+    { MEMCACHED_BEHAVIOR_IO_MSG_WATERMARK, "_io_msg_watermark" },
+    { MEMCACHED_BEHAVIOR_IO_BYTES_WATERMARK, "_io_bytes_watermark" },
+    { MEMCACHED_BEHAVIOR_IO_KEY_PREFETCH, "_io_key_prefetch" },
+    { MEMCACHED_BEHAVIOR_HASH_WITH_PREFIX_KEY, "_hash_with_prefix_key" },
+    { MEMCACHED_BEHAVIOR_NOREPLY, "_noreply" },
+    { MEMCACHED_BEHAVIOR_AUTO_EJECT_HOSTS, "_auto_eject_hosts" },
+    { MEMCACHED_BEHAVIOR_NUMBER_OF_REPLICAS, "_number_of_replicas" },
     { 0, NULL }
 };
 
@@ -147,6 +155,7 @@ static PyObject *PylibMC_Client_get_behaviors(PylibMC_Client *);
 static PyObject *PylibMC_Client_set_behaviors(PylibMC_Client *, PyObject *);
 static PyObject *PylibMC_Client_flush_all(PylibMC_Client *, PyObject *, PyObject *);
 static PyObject *PylibMC_Client_disconnect_all(PylibMC_Client *);
+static PyObject *PylibMC_Client_clone(PylibMC_Client *);
 static PyObject *PylibMC_ErrFromMemcached(PylibMC_Client *, const char *,
         memcached_return);
 static PyObject *_PylibMC_Unpickle(const char *, size_t);
@@ -189,6 +198,9 @@ static PyMethodDef PylibMC_ClientType_methods[] = {
         METH_VARARGS|METH_KEYWORDS, "Flush all data on all servers."},
     {"disconnect_all", (PyCFunction)PylibMC_Client_disconnect_all, METH_NOARGS,
         "Disconnect from all servers and reset own state."},
+    {"clone", (PyCFunction)PylibMC_Client_clone, METH_NOARGS,
+        "Clone this client entirely such that it is safe to access from "
+        "another thread. This creates a new connection."},
     {NULL, NULL, 0, NULL}
 };
 /* }}} */
