@@ -6,12 +6,13 @@ minor differences. If you should happen to spot any, file a bug!
 >>> import pylibmc
 >>> mc = pylibmc.Client(["127.0.0.1"])
 >>> b = mc.behaviors
->>> list(sorted(b.keys()))  # doctest: +NORMALIZE_WHITESPACE
-['binary_protocol', 'buffer_requests', 'cache_lookups', 'connect_timeout',
- 'distribution', 'hash', 'ketama', 'ketama_hash', 'ketama_weighted',
- 'no block', 'poll_timeout', 'rcv_timeout', 'retry_timeout',
- 'server_failure_limit', 'snd_timeout', 'socket recv size', 'socket send size',
- 'sort_hosts', 'support_cas', 'tcp_nodelay', 'verify_key']
+>>> ks = list(sorted(k for k in b.keys() if not k.startswith("_")))
+>>> ks  # doctest: +NORMALIZE_WHITESPACE
+['buffer_requests', 'cache_lookups', 'connect_timeout', 'distribution', 'hash',
+ 'ketama', 'ketama_hash', 'ketama_weighted', 'no block', 'poll_timeout',
+ 'rcv_timeout', 'retry_timeout', 'server_failure_limit', 'snd_timeout',
+ 'socket recv size', 'socket send size', 'sort_hosts', 'support_cas',
+ 'tcp_nodelay', 'verify_key']
 >>> b["hash"]
 'default'
 >>> b["hash"] = 'fnv1a_32'
@@ -24,7 +25,7 @@ minor differences. If you should happen to spot any, file a bug!
 import _pylibmc
 
 __all__ = ["hashers", "distributions", "Client"]
-__version__ = "0.7.4"
+__version__ = _pylibmc.__version__
 
 hashers, hashers_rvs = {}, {}
 distributions, distributions_rvs = {}, {}
@@ -53,12 +54,14 @@ class BehaviorDict(dict):
         self.client.set_behaviors(self.copy())
 
 class Client(_pylibmc.client):
-    def __init__(self, servers, *args, **kwds):
+    def __init__(self, servers, binary=False):
         """Initialize a memcached client instance.
 
         This connects to the servers in *servers*, which will default to being
         TCP servers. If it looks like a filesystem path, a UNIX socket. If
         prefixed with `udp:`, a UDP connection.
+
+        If *binary* is True, the binary memcached protocol is used.
         """
         addr_tups = []
         for server in servers:
@@ -80,7 +83,7 @@ class Client(_pylibmc.client):
             else:
                 stype = _pylibmc.server_type_tcp
             addr_tups.append((stype, addr, port))
-        super(Client, self).__init__(addr_tups)
+        super(Client, self).__init__(servers=addr_tups, binary=binary)
 
     def get_behaviors(self):
         """Gets the behaviors from the underlying C client instance.
@@ -112,6 +115,86 @@ class Client(_pylibmc.client):
 
     behaviors = property(get_behaviors, set_behaviors)
 
+from contextlib import contextmanager
+
+class ClientPool(list):
+    """Client pooling helper.
+
+    This is mostly useful in threaded environments, because a client isn't
+    thread-safe at all. Instead, what you want to do is have each thread use
+    its own client, but you don't want to reconnect these all the time.
+
+    The solution is a pool, and this class is a helper for that.
+
+    >>> mc = Client(["127.0.0.1"])
+    >>> pool = ClientPool()
+    >>> pool.fill(mc, 4)
+    >>> with pool.reserve() as mc:
+    ...     mc.set("hi", "ho")
+    ...     mc.delete("hi")
+    ... 
+    """
+
+    @contextmanager
+    def reserve(self):
+        """Reserve a client, and put it back into the pool when done."""
+        mc = self.pop()
+        try:
+            yield mc
+        finally:
+            self.append(mc)
+
+    def fill(self, mc, n_slots):
+        """Fill *n_slots* of the pool with clones of *mc*."""
+        for i in xrange(n_slots):
+            self.append(mc.clone())
+
+class ThreadMappedPool(dict):
+    """Much like the *ClientPool*, helps you with pooling.
+
+    In a threaded environment, you'd most likely want to have a client per
+    thread. And there'd be no harm in one thread keeping the same client at all
+    times. So, why not map threads to clients? That's what this class does.
+
+    If a client is reserved, this class checks for a key based on the current
+    thread, and if none exists, clones the master client and inserts that key.
+
+    >>> mc = Client(["127.0.0.1"])
+    >>> pool = ThreadMappedPool(mc)
+    >>> with pool.reserve() as mc:
+    ...     mc.set("hi", "ho")
+    ...     mc.delete("hi")
+    ... 
+    """
+
+    def __new__(cls, master):
+        return super(ThreadMappedPool, cls).__new__(cls)
+
+    def __init__(self, master):
+        self.master = master
+
+    @contextmanager
+    def reserve(self):
+        """Reserve a client.
+
+        Creates a new client based on the master client if none exists for the
+        current thread.
+        """
+        key = threading.current_thread().name
+        mc = self.pop(key, None)
+        if mc is None:
+            mc = self.master.clone()
+        try:
+            yield mc
+        finally:
+            self[key] = mc
+
+# This makes sure ThreadMappedPool doesn't exist with non-thread Pythons.
+try:
+    import threading
+except ImportError:
+    del ThreadMappedPool
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
