@@ -1,6 +1,8 @@
 """Tests. They want YOU!!
 
 Basic functionality.
+>>> _pylibmc.__version__ == pylibmc.__version__
+True
 >>> c = _pylibmc.client([test_server])
 >>> c.set("test_key", 123)
 True
@@ -56,7 +58,7 @@ and yeah, so you know...
 True
 >>> c.get("hi")
 'steven'
->>> sleep(1.1)
+>>> sleep(2.1)
 >>> c.get("hi")
 >>> c.set("hi", "loretta", int(time()) + 2)
 True
@@ -114,12 +116,32 @@ True
 >>> c.get("tengil").bar == bla.bar
 True
 
+Cloning (ethically, I don't know about it.)
+>>> c is not c.clone()
+True
+>>> c2 = c.clone()
+>>> c.set("test", "hello")
+True
+>>> c2.get("test")
+'hello'
+>>> c2.delete("test")
+True
+>>> del c2
 
 Behaviors.
 >>> c.set_behaviors({"tcp_nodelay": True, "hash": 6})
 >>> list(sorted((k, v) for (k, v) in c.get_behaviors().items()
 ...             if k in ("tcp_nodelay", "hash")))
 [('hash', 6), ('tcp_nodelay', 1)]
+
+Binary protocol!
+>>> c = _pylibmc.client([test_server], binary=True)
+>>> c.set("hello", "world")
+True
+>>> c.get("hello")
+'world'
+>>> c.delete("hello")
+True
 """
 
 # Used to test pickling.
@@ -170,6 +192,7 @@ def is_alive(addr):
 if __name__ == "__main__":
     print "Starting tests with _pylibmc at", _pylibmc.__file__
     print "Reported libmemcached version:", _pylibmc.libmemcached_version
+    print "Reported pylibmc version:", _pylibmc.__version__
     if not is_alive(test_server):
         raise SystemExit("Test server (%r) not alive." % (test_server,))
     import doctest
